/*!
 * Ghostery for Firefox
 * http://www.ghostery.com/
 *
 * Copyright 2013 EVIDON, Inc. All rights reserved.
 * See ghostery_eula.txt for license.
 */

var _ = require('vendor/underscore-1.4.3'),
	conf = require('conf').load,
	utils = require('utils'),
	UpdatableMixin = require('updatable').load;

var log = utils.log;

function BugDb() {
	this.type = 'bugs';

	function buildDb(bugs, version) {
		var i, num_bugs,
			bug,
			pattern,
			apps = {},
			cats = {},
			bugs_map = {},
			first_party_map = {},
			num_selected_patterns = 0,
			patterns_arr = [],
			HASH = {
				host: {},
				host_path: {},
				path: {},
				regex: {}
			};

		// TODO review this code
		// Remove any existing user-created bugs
		for (i = bugs.length - 1; i > -1; i--) {
			if (bugs[i].type == 'user_created') {
				bugs.splice(i, 1);
			} else {
				break;
			}
		}

		var userBugs = processUserCreatedBugs();
		for (i = 0; i < userBugs.length; i++) {
			bugs.push(userBugs[i]);
		}

		for (i = 0, num_bugs = bugs.length; i < num_bugs; i++) {
			bug = bugs[i];
			pattern = bug.pattern;

			bugs_map[bug.id] = {
				aid: bug.aid,
				name: bug.name
			};

			apps[bug.aid] = {
				name: bug.name,
				cat: bug.type
			};

			if (bug.classification == 4 || bug.classification === undefined) {
				HASH.regex[bug.id] = new RegExp(pattern, 'i');
			} else {
				(unwrap(pattern) || []).forEach(function (pattern) {
					switch (bug.classification) {
					case 1:
					case 5:
						addHost(HASH.host, pattern.replace(/(\/.*)|(^\.)/g, ''), bug.id);
						if (bug.classification == 1) { break; }

					case 2:
					case 6:
						if (bug.classification != 5) {
							addHost(HASH.host_path, pattern.replace(/(\/.*)|(^\.)/g, ''), bug.id, pattern.replace(/^.*?\//, ''));
						}

						if (bug.classification == 2) { break; }

					case 3:
						// HASH.path[pattern.replace(/^\//, '')] = bug.id;
						HASH.path[pattern] = bug.id;
						break;
					}
				});
			}

			patterns_arr.push(bug.pattern);

			if (bug.hasOwnProperty('firstPartyUrls')) {
				first_party_map[bug.id] = bug.firstPartyUrls;
			}

			if (!cats.hasOwnProperty(bug.type)) {
				cats[bug.type] = {};
			}
			cats[bug.type][bug.aid] = true;

			if (conf.selected_app_ids.hasOwnProperty(bug.aid)) {
				num_selected_patterns++;
			}
		}

		return {
			JUST_UPDATED_WITH_NEW_TRACKERS: false,

			allSelected: (num_selected_patterns == patterns_arr.length),
			noneSelected: num_selected_patterns === 0,

			apps: apps,
			bugs: bugs_map,
			cats: cats,
			firstPartyExceptions: first_party_map,
			HASH: HASH,
			version: version
		};
	}

	function unwrap(pattern) {
		var combined = [],
			first,
			last,
			parts,
			i;

		// TODO comment
		if (pattern.replace(/(\\\.|\\\/|\\\?|\\\-)/g, '').match(/^[A-Za-z0-9;\-_]+$/)) {
			combined.push(pattern.replace(/\\/g, ''));

		// TODO comment
		} else if (pattern.split('(').length == 2) {
			first = pattern.substring(0, pattern.indexOf('(')),
			last = pattern.substring(pattern.lastIndexOf(')') + 1, pattern.length),
			parts = pattern.substring(pattern.indexOf('(') + 1, pattern.lastIndexOf(')')).split('|');

			// if this is an optional group, end all processing
			if (last != '?' && last.charAt(0) != '?') {
				for (i = 0; i < parts.length; i++) {
					combined.push((first + parts[i] + last).replace(/\\/g, ''));
				}
			}
		}

		// sanity check
		for (i = 0; i < combined.length; i++) {
			if (!combined[i].replace(/(\.|\/|\?)/g, '').match(/^[A-Za-z0-9;\-_]+$/)) {
				return false;
			}
		}

		return combined || false;
	}

	function addHost(root, src_host, bug_id, src_path) {
		var host_rev_arr = src_host.split('.').reverse(),
			part,
			node = root,
			i;

		for (i = 0; i < host_rev_arr.length; i++) {
			part = host_rev_arr[i];

			// if the node does not have a matching child, add it
			if (!node.hasOwnProperty(part)) {
				node[part] = {};
			}

			// if we are on the last "part"
			if (i == host_rev_arr.length - 1) {

				// handle path
				if (src_path !== undefined) {
					if (!node[part].hasOwnProperty('$')) {
						node[part].$ = {};
					}

					node[part].$[src_path] = bug_id;

				// handle host only
				} else {
					node[part].$ = bug_id;
				}
			}

			// advance node
			node = node[part];
		}
	}

	function updateNewAppIds(new_bugs, old_bugs) {
		log('updating newAppIds ...');

		var new_app_ids = _.difference(
			_.pluck(new_bugs, 'aid'),
			_.pluck(old_bugs, 'aid')
		);

		utils.prefs('newAppIds', new_app_ids);

		return new_app_ids;
	}

	function processUserCreatedBugs() {
		var inputtedBugs = [],
			userBugs = [];

		try {
			inputtedBugs = utils.prefs('user_bugs');
		} catch (e) {}

		for (var i = 0; i < inputtedBugs.length; i++) {
			var bug = inputtedBugs[i];
			if (!bug.pattern) {
				continue;
			}

			bug.userCreated = (bug.type == 'user_created');

			if (!bug.hasOwnProperty('name')) {
				bug.name = 'User Created Bug ' + (i + 1);
			}

			if (!bug.hasOwnProperty('aid')) {
				bug.aid = 10000 + i;
			}

			if (!bug.hasOwnProperty('cid')) {
				bug.cid = 10000 + i;
			}

			if (!bug.hasOwnProperty('id')) {
				bug.id = 10000 + i;
			}

			userBugs.push(bug);
		}

		return userBugs;
	}

	this.processList = function (bugs) {
		var db;

		log('processing bugs ...');

		try {
			db = buildDb(bugs.bugs, bugs.bugsVersion);
		} catch (e) {}

		if (!db) {
			return false;
		}

		log('processed');

		var old_bugs = utils.prefs('bugs');

		// there is an older bugs object and the versions are different
		// TODO should be doing > instead of != once we convert all versions to timestamps
		if (old_bugs && bugs.bugsVersion != old_bugs.bugsVersion) {
			var new_app_ids = updateNewAppIds(bugs.bugs, old_bugs.bugs);

			if (conf.block_by_default) {
				log('applying block-by-default ...');

				_.each(new_app_ids, function (app_id) {
					conf.selected_app_ids[app_id] = 1;
				});
			}

			if (new_app_ids.length) {
				db.JUST_UPDATED_WITH_NEW_TRACKERS = true;
			}
		}

		this.db = db;
		utils.prefs('bugs', bugs);

		return true;
	};

	_.extend(this, UpdatableMixin);
}

exports.load = new BugDb();
