'use strict';

const { unload } = require("unload+");
const { listen } = require("listen");
const winUtils = require("window-utils");
const tabsLib = require('sdk/tabs/helpers');
const runtime = require('sdk/system/runtime');
const Cc = require('chrome').Cc;
const Ci = require('chrome').Ci;

// custom

const utils = require('ghostery/utils');
const prefs = utils.prefs;

const browserURL = "chrome://browser/content/browser.xul";
const NS_XUL = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

exports.ToolbarButton = function ToolbarButton(options) {
  var unloaders = [],
      toolbarID = "",
      insertbefore = "",
      destroyed = false,
      destoryFuncs = [];

  var delegate = {
    onTrack: function (window) {
 			// remembering positioing preferences if they get reset by another addon
    	window.addEventListener("aftercustomization", function (e) {
					var doc = e.target.ownerDocument,
					 badge = doc.getElementById(options.id);

					if (!!badge) {
						// if badge is present, save the current toolbar
						prefs('badgeLocation', badge.parentElement.id);

						if (badge.nextSibling) {
							prefs('badgeLocationInsertBefore', badge.nextSibling.id);
						} else {
							prefs('badgeLocationInsertBefore', false);
						}
					} else {
						prefs('badgeLocationInsertBefore', false);
						prefs('badgeLocation', 'hidden');
					}
				});

			// normal: [object ChromeWindow],[object XULDocument]
			// fucked: [object XrayWrapper [object Window]],[object XrayWrapper [object HTMLDocument]]
			// Apparently you get fucked windows when its a JS generated pop-up
			var nonstandardWindow = false;
      if (browserURL != window.location) {
				var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
				window = wm.getMostRecentWindow(null);

				nonstandardWindow = true;
      }

      if (destroyed)
        return;

      let doc = window.document;
      let $ = function(id) doc.getElementById(id);

      options.tooltiptext = options.tooltiptext || '';

      // create toolbar button
      let tbb = doc.createElementNS(NS_XUL, "toolbarbutton");
      let tbbContainer = doc.createElementNS(NS_XUL, "toolbaritem");
      let tbbStack = doc.createElementNS(NS_XUL, "stack");
      let tbbBadge = doc.createElementNS(NS_XUL, "label");

      tbbBadge.setAttribute('id', "ghostery-button-badge");
      var binding =
        '<bindings xmlns="http://www.mozilla.org/xbl">' +
          '<binding id="id">' +
            '<resources>' +
              '<stylesheet src="' + options.stylesheet + '"/>' +
            '</resources>' +
          '</binding>' +
        '</bindings>';
      tbbBadge.style.MozBinding = 'url("data:text/xml;charset=utf-8,' + doc.defaultView.encodeURIComponent(binding) + '")';

      tbbContainer.setAttribute("id", options.id);
      tbbContainer.setAttribute("label", options.label);
      tbbContainer.setAttribute('tooltiptext', options.tooltiptext);
      tbb.setAttribute("type", "button");
      if (options.image) tbb.setAttribute("image", options.image);
      
      if (!nonstandardWindow) {
      	tbb.setAttribute("class", "toolbarbutton-1 chromeclass-toolbar-additional");
      } else {
      	tbb.setAttribute("class", "toolbarbutton-1");
      }

      tbb.setAttribute("id", "ghostery-button");

      tbb.addEventListener("command", function() {
        if (options.onCommand)
          options.onCommand({}); // TODO: provide something?

        if (options.panel) {
          options.panel.show(tbbContainer);
        }
      }, true);

      tbbBadge.addEventListener("click", function() {
        if (options.onCommand)
          options.onCommand({}); // TODO: provide something?

        if (options.panel) {
          options.panel.show(tbbContainer);
        }
      }, true);

      positionBadge(tbbBadge);

      // append button and badge to stack
      tbbStack.appendChild(tbb);
      tbbStack.appendChild(tbbBadge);
      // append stack to the button container
      tbbContainer.appendChild(tbbStack);

      // add toolbarbutton to palette
      try {
      	($("navigator-toolbox") || $("mail-toolbox")).palette.appendChild(tbbContainer);
    	} catch (e) { }

      // find a toolbar to insert the toolbarbutton into
      if (prefs('badgeLocation')) {
        var tb = $(prefs('badgeLocation'));
      } else {
      	tb = $('nav-bar');
      }

      if (!tb) {
        var tb = toolbarbuttonExists(doc, options.id);
      }

      // found a toolbar to use?
      if (tb) {
        let b4;

        // find the toolbarbutton to insert before
        if (prefs('badgeLocationInsertBefore')) {
          b4 = $(prefs('badgeLocationInsertBefore'));
        }

        if (!b4) {
          let currentset = tb.getAttribute("currentset").split(",");
          let i = currentset.indexOf(options.id) + 1;

          // was the toolbarbutton id found in the curent set?
          if (i > 0) {
            let len = currentset.length;
            // find a toolbarbutton to the right which actually exists
            for (; i < len; i++) {
              b4 = $(currentset[i]);
              if (b4) break;
            }
          }
        }

        tb.insertItem(options.id, b4, null, false);
        tb.setAttribute("currentset", tb.currentSet);

        try {
          // adding keyset
          var keyset = doc.createElement('keyset'), keyelem = doc.createElement('key');
          keyset.id = 'ghostery-keyset';

          keyelem.setAttribute('id', 'ghostery-finder-key');
          keyelem.setAttribute('oncommand', "void(0);");
          keyelem.addEventListener("command", function() { tbb.click(); }, true);
          
          keyelem.setAttribute('key', 'g');
          keyelem.setAttribute('modifiers', 'alt control');
          
          keyset.appendChild(keyelem);
          $('main-window').appendChild(keyset);
        } catch (err) {
          // wrong main window.
        }

				if (!nonstandardWindow) {
        	doc.persist(tb.id, "currentset");
        }
      }

      var saveTBNodeInfo = function(e) {
        toolbarID = tbbContainer.parentNode.getAttribute("id") || "";
        insertbefore = (tbbContainer.nextSibling || "")
            && tbbContainer.nextSibling.getAttribute("id").replace(/^wrapper-/i, "");
      };

      window.addEventListener("aftercustomization", saveTBNodeInfo, false);

      // add unloader to unload+'s queue
      var unloadFunc = function() {
      	try {
        	tbbContainer.parentNode.removeChild(tbbContainer);
        	window.removeEventListener("aftercustomization", saveTBNodeInfo, false);
        } catch (e) {
					// tripped when window never had successful container injecttion
				}
      };

      var index = destoryFuncs.push(unloadFunc) - 1;

      listen(window, window, "unload", function() {
        destoryFuncs[index] = null;
      }, false);

      unloaders.push(unload(unloadFunc, window));
    },
    onUntrack: function (window) {}
  };

  var tracker = winUtils.WindowTracker(delegate);

  function setIcon(tab_id, pathToImage) {
    for (var window in winUtils.windowIterator()) {
      if ('chrome://browser/content/browser.xul' != window.location) { continue; }
      if (tab_id != tabsLib.getTabForWindow(window.content).id) { continue; }
      
      var button = window.document.getElementById('ghostery-button');
      if (!button) { continue; }
      button.setAttribute('image', pathToImage);
    }
  }

  function setBadge(tab_id, value) {
    for (var window in winUtils.windowIterator()) {
      if ('chrome://browser/content/browser.xul' != window.location) { continue; }
      if (tab_id != tabsLib.getTabForWindow(window.content).id) { continue; }

      var badge = window.document.getElementById('ghostery-button-badge');
      if (!badge) { continue; }

      if (value || value == '0') {
        badge.style.visibility = 'visible';
      } else {
        badge.style.visibility = 'hidden';
      }

      badge.setAttribute('value', value);
    }
  }

  return {
    destroy: function() {
      if (destroyed) return;
      destroyed = true;

      if (options.panel)
        options.panel.destroy();

      // run unload functions
      destoryFuncs.forEach(function(f) f && f());
      destoryFuncs.length = 0;

      // remove unload functions from unload+'s queue
      unloaders.forEach(function(f) f());
      unloaders.length = 0;
    },
    moveTo: function(pos) {

      if (destroyed) return;

      // record the new position for future windows
      toolbarID = pos.toolbarID;
      insertbefore = pos.insertbefore;

      // change the current position for open windows
      for each (var window in winUtils.windowIterator()) {
        if (browserURL != window.location) return;

        let doc = window.document;
        let $ = function (id) doc.getElementById(id);

        // if the move isn't being forced and it is already in the window, abort
        if (!pos.forceMove && $(options.id)) return;

        var tb = $(toolbarID);
        var b4 = $(insertbefore);

        // TODO: if b4 dne, but insertbefore is in currentset, then find toolbar to right

        if (tb) {
          tb.insertItem(options.id, b4, null, false);
          tb.setAttribute("currentset", tb.currentSet);
          doc.persist(tb.id, "currentset");
        }
      };
    },
    get label() options.label,
    set label(value) {
      options.label = value;
      getToolbarButtons(function(tbb) {
        tbb.label = value;
      }, options.id);
      return value;
    },
    setIcon: setIcon,
    setBadge: setBadge,
    get image() options.image,
    set image(value) setIcon({image: value}),
    get tooltiptext() options.tooltiptext,
    set tooltiptext(value) {
      options.tooltiptext = value;
      getToolbarButtons(function(tbb) {
        tbb.setAttribute('tooltiptext', value);
      }, options.id);
    },
  };
};

function getToolbarButtons(callback, id) {
  let buttons = [];
  for each (var window in winUtils.windowIterator()) {
    if (browserURL != window.location) continue;
    let tbb = window.document.getElementById(id);
    if (tbb) buttons.push(tbb);
  }
  if (callback) buttons.forEach(callback);
  return buttons;
}

function toolbarbuttonExists(doc, id) {
  var toolbars = doc.getElementsByTagNameNS(NS_XUL, "toolbar");
  for (var i = toolbars.length - 1; ~i; i--) {
    if ((new RegExp("(?:^|,)" + id + "(?:,|$)")).test(toolbars[i].getAttribute("currentset")))
      return toolbars[i];
  }
  return false;
}

function positionBadge(buttonBadge) {
  var os = runtime.OS,
    right,
    bottom;
  
  switch (os) {
  case 'Darwin':
    right = '-5';
    bottom = '-5';
    break;
  case 'WINNT':
    right = '-1';
    bottom = '3';
    break;
  case 'Linux':
    right = '-3';
    bottom = '0';
    break;
  default:
    right = '-5';
    bottom = '-5';
  }
  
  buttonBadge.setAttribute('right', right);
  buttonBadge.setAttribute('bottom', bottom);
}
